package envoy.client.data.shortcuts;

import java.util.*;
import java.util.Map.Entry;

import javafx.scene.input.KeyCombination;

import envoy.client.ui.SceneContext.SceneInfo;

/**
 * Contains all KeyBoardshotcuts used throughout the application.
 *
 * @author Leon Hofmeister
 * @since Envoy Client v0.3-beta
 */
public final class GlobalKeyShortcuts {

	/**
	 * Helper class for the Entry interface.
	 *
	 * @author Leon Hofmeister
	 * @since Envoy Client v0.3-beta
	 */
	public final class KeyCombinationAction implements Entry<KeyCombination, Runnable> {

		private final KeyCombination	keys;
		private final Runnable			action;

		/**
		 * Creates a new {@code KeyCombinationAction}.
		 *
		 * @param keys   the keys to press to perform the given action
		 * @param action the action to perform
		 * @since Envoy Client v0.3-beta
		 */
		public KeyCombinationAction(KeyCombination keys, Runnable action) {
			if (keys == null || action == null) throw new NullPointerException("Cannot create key combination action");
			this.keys	= keys;
			this.action	= action;
		}

		@Override
		public KeyCombination getKey() { return keys; }

		@Override
		public Runnable getValue() { return action; }

		@Override
		public Runnable setValue(Runnable none) {
			throw new UnsupportedOperationException("Reassignment of keyboard shortcut actions is not supported");
		}

	}

	private final EnumMap<SceneInfo, Collection<KeyCombinationAction>> shortcuts = new EnumMap<>(SceneInfo.class);

	private static GlobalKeyShortcuts instance = new GlobalKeyShortcuts();

	private GlobalKeyShortcuts() {
		for (final var sceneInfo : SceneInfo.values())
			shortcuts.put(sceneInfo, new HashSet<KeyCombinationAction>());
	}

	/**
	 * @return the instance of global keyboard shortcuts.
	 * @since Envoy Client v0.3-beta
	 */
	public static GlobalKeyShortcuts getInstance() { return instance; }

	/**
	 * Adds the given keyboard shortcut and its action to all scenes.
	 *
	 * @param keys   the keys to press to perform the given action
	 * @param action the action to perform
	 * @since Envoy Client v0.3-beta
	 */
	public void add(KeyCombination keys, Runnable action) {
		final var keyCombinationAction = new KeyCombinationAction(keys, action);
		shortcuts.values().forEach(collection -> collection.add(keyCombinationAction));
	}

	/**
	 * Adds the given keyboard shortcut and its action to all scenes that are not
	 * part of exclude.
	 *
	 * @param keys    the keys to press to perform the given action
	 * @param action  the action to perform
	 * @param exclude the scenes that should be excluded from receiving this
	 *                keyboard shortcut
	 * @since Envoy Client v0.3-beta
	 */
	public void addForNotExcluded(KeyCombination keys, Runnable action, SceneInfo... exclude) {

		// Computing the remaining sceneInfos
		final var	include	= new SceneInfo[SceneInfo.values().length - exclude.length];
		int			index	= 0;
		outer:
		for (final var sceneInfo : SceneInfo.values()) {
			for (final var excluded : exclude)
				if (sceneInfo.equals(excluded)) continue outer;
			include[index++] = sceneInfo;
		}

		// Adding the action to the remaining sceneInfos
		final var keyCombinationAction = new KeyCombinationAction(keys, action);
		for (final var sceneInfo : include)
			shortcuts.get(sceneInfo).add(keyCombinationAction);
	}

	/**
	 * Returns all stored keyboard shortcuts for the given scene constant
	 *
	 * @param sceneInfo the currently loading scene
	 * @return all stored keyboard shortcuts for this scene
	 * @since Envoy Client v0.3-beta
	 */
	public Collection<KeyCombinationAction> getKeyboardShortcuts(SceneInfo sceneInfo) { return shortcuts.get(sceneInfo); }
}
